<?php


namespace OrganizationsManagement\Controller;

use Exception;
use OrganizationsManagement\Traits\TResponse;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\InstanceProvider;
use \OrganizationsManagement\Service\Slacliente as SlaclienteService;

class Slacliente
{
    use TResponse;

    private $slaclienteService;

    public function __construct(SlaclienteService $slaclienteService){
        $this->slaclienteService = $slaclienteService;
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function findSlacliente(DTO\Request $request, DTO\Response $response){
        try{
            $cdcliente = $request->getParameter('CDCLIENTE');
            $data = $this->slaclienteService->findSlacliente($cdcliente);
            $this->sendResponse($data, 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function updateSlacliente(DTO\Request $request, DTO\Response $response)
    {
        try{
            $this->slaclienteService->updateSlacliente($request->getParameters());
            $this->sendResponse([], 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function deleteSlacliente(DTO\Request $request, DTO\Response $response)
    {
        try{
            $this->slaclienteService->validationsPermissisonDelete($request->getParameters()[0]['CDCLIENTE']);
            $this->slaclienteService->deleteSlacliente($request->getParameters());
            $this->sendResponse([], 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    /**
     * @param DTO\Request $request
     * @param DTO\Response $response
     * @throws Exception
     */
    public function saveSlacliente(DTO\Request $request, DTO\Response $response)
    {
        try{
            $data = $request->getParameters();
            $cdcliente = $data["CDCLIENTE"];
            $cdassunto = $data["CDASSUNTO"];
            $cdgrupfase = $data["CDGRUPFASE"];
            $idprazosla = $data["IDPRAZOSLA"];
            $qtdiassla = $data["QTDIASSLA"];
            $dtiniciosla = $data["DTINICIOSLA"];
            $this->slaclienteService->saveSlacliente($cdcliente, $cdassunto, $cdgrupfase, $idprazosla, $qtdiassla, $dtiniciosla);
            $this->sendResponse([], 200);
        }catch(Exception $ex){
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
