import { OMInstance } from '@/services/OMInstance';
import ApiSlacliente from '@/services/api/Slacliente';
import ApiSlaclienteParam from '@/services/api/SlaclienteParam';
import ApiSlaclienteAlert from '@/services/api/SlaclienteAlert';
import { Metadata } from '@zeedhi/core';
import { Modal, ModalService, Select, TextInput, DialogService } from '@zeedhi/common';
import { OmDialogService } from '@/services/OmDialogService';
import Messages from '@/services/Messages';
import ScreenService from '@/services/ScreenService';
import Store from '../store/index';

export class SlaclienteController {
	public clientSelected = false;

	public alertGridDisabled = true;

	get denieClienteSelected() {
		return !this.clientSelected;
	}

	public slaDefaultClient = false;

	/** **************************************************************************
	 *************************** Section Grid Cliente ****************************
	 *************************************************************************** */

	/** properties */
	private cdclienteTeknisa = 'J26269316000177';

	private clienteGridCurrentRow: any = {};

	private dataSourceIdParamSla = [
		{
			id: 'P',
			name: 'SLA Padrão Teknisa',
		},
		{
			id: 'C',
			name: 'SLA Cliente',
		},
	];

	private dataSourceIdControlaSla = [
		{
			id: 'S',
			name: 'Sim (cliente possui contrato)',
		},
		{
			id: 'I',
			name: 'Sim (apenas controle Teknisa)',
		},
		{
			id: 'N',
			name: 'Não',
		},
	];

	/** Accessors */

	get nomeFatasia() {
		return this.clienteGridCurrentRow.NMFANTCLIE || '';
	}

	public editableColIdParamSla(params: any) {
		if (params.component) {
			const cellControlSla = Metadata.getInstance<Select>(`ID_CONTROLA_SLA_editable_${params.row.CDCLIENTE}`);
			const cellParamSla = Metadata.getInstance<Select>(`ID_PARAM_SLA_editable_${params.row.CDCLIENTE}`);
			if (params.component.value === 'N' || !params.component.value) {
				cellParamSla.value = '';
				cellParamSla.disabled = true;
			} else if ((cellControlSla.value === 'N' || !cellControlSla.value) && params.column.name === 'ID_PARAM_SLA') {
				cellParamSla.value = '';
				cellParamSla.disabled = true;
			} else {
				cellParamSla.disabled = false;
			}
		}
	}

	/** Actions  */

	public async reloadClienteGrid(): Promise<any> {
		const { datasource } = OMInstance.getInstanceTekGrid('clienteGrid');
		try {
			const response = await datasource.get();
			this.clienteGridCurrentRow = {};
			this.slaclienteGridCurrentRow = {};
			return response;
		} catch (error) {
			return false;
		}
	}

	public async saveClienteGrid(params: any) {
		params.event.preventDefault();
		try {
			await ApiSlaclienteParam.updateSlaclienteParam(params.component.getEditedRows());
			OmDialogService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
			params.component.editing = false;
			params.component.datasource.get();
		} catch (error) {
			OmDialogService.showErrorBackend(error);
		}
	}

	/** Events  */

	public async onRowClickedClienteGrid(params: any) {
		if (!params.component.editing) {
			try {
				const { datasource } = OMInstance.getInstanceTekGrid('slaclienteGrid');
				datasource.filter.CDCLIENTE = params.row.CDCLIENTE;
				await Store.commit('Cliente/SET_CLIENTE', params.row);
				await Store.dispatch('Cliente/setSlacliente', {});
				this.setClienteGridCurrentRow(params.row);
				this.slaclienteGridCurrentRow = {};
				if (params.row.ID_CONTROLA_SLA !== 'N') {
					if (params.row.ID_PARAM_SLA === 'P') {
						datasource.filter.CDCLIENTE = this.cdclienteTeknisa;
					} else {
						this.slaDefaultClient = true;
					}
					await datasource.get();
					datasource.currentRow = await datasource.data[0];
					await this.onRowClickedSlaclienteGrid(datasource.data[0]);
					this.clientSelected = true;
				}
			} catch (e) {
				OmDialogService.showErrorBackend(e);
			}
		}
	}

	/** methods */

	public setClienteGridCurrentRow(row: object) {
		this.clienteGridCurrentRow = row;
	}

	/** *************************************************************************************
	 *************************** Section Grid Sla Cliente **********************************
	 ************************************************************************************** */

	/** properties */

	public slaclienteGridCurrentRow: any = {};

	private dataSourceCdgrupofase = [
		{
			id: '13',
			name: 'Especificação',
		},
		{
			id: '30',
			name: 'Implementação',
		},
	];

	/** Accessors */

	get heightSlaclienteGrid() {
		const aux = ((window.innerHeight * 0.55) - 96) - 178;
		if (aux < 100) {
			return '90px';
		}
		return `${aux}px`;
	}

	set heightSlaclienteGrid(_value: any) {
		// do nothing ();
	}

	public isEditableColIdPrazoId(param: any) {
		if (this.slaDefaultClient) {
			return param.row.CDGRUPFASE !== '13' && this.clienteGridCurrentRow.ID_PARAM_SLA !== 'P';
		}
		return false;
	}

	public isEditableColDtiniciosla() {
		return this.clienteGridCurrentRow.ID_PARAM_SLA !== 'P';
	}

	/** Actions  */

	public async reloadSlaclienteGrid() {
		const { datasource } = OMInstance.getInstanceTekGrid('slaclienteGrid');
		try {
			datasource.filter.CDCLIENTE = Store.getters['Cliente/getCdcliente'];
			await datasource.get();
		} catch (e) {
			OmDialogService.showErrorBackend(e);
		}
	}

	public async saveSlaclienteGrid(params: any) {
		params.event.preventDefault();
		const edditedRows = params.component.getEditedRows();
		edditedRows.forEach((row) => {
			if (row.DTINICIOSLA.split('-').length > 1) {
				const dateSplited = row.DTINICIOSLA.split('-');
				const convertedDate = `${dateSplited[2]}/${dateSplited[1]}/${dateSplited[0]}`;
				row.DTINICIOSLA = convertedDate;
			}
		});
		try {
			await ApiSlacliente.updateSlacliente(edditedRows);
			OmDialogService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
			params.component.editing = false;
			params.component.datasource.get();
			this.resetSlaAlertGrid();
		} catch (error) {
			OmDialogService.showErrorBackend(error);
		}
	}

	public async deleteSlaclienteGrid(params: any) {
		params.event.preventDefault();
		if (this.slaDefaultClient) {
			const paramsAux = { data: params.component.selectedRows };
			try {
				await ApiSlacliente.deleteSlacliente(paramsAux);
				OmDialogService.showSuccess(Messages.SUCCESS_DELETE_DEAFULT);
				await this.reloadSlaclienteGrid();
				params.component.selectedRows = [];
			} catch (e) {
				OmDialogService.showErrorBackend(e);
			}
		} else {
			DialogService.show({
				name: 'error-dialog',
				type: 'error',
				title: 'Erro',
				text: Messages.ERROR_DEFAULT_SLA_PARAM,
			});
		}
	}

	/** Events  */

	public backButtonSlaClient() {
		this.clientSelected = false;
		this.slaDefaultClient = false;
		this.reloadClienteGrid();
		this.resetSlaAlertGrid();
	}

	public async onRowClickedSlaclienteGrid(params: any) {
		if (params) {
			let paramsFiltered = params;
			if (!paramsFiltered.row) {
				paramsFiltered = { row: params };
			}
			if (Object.keys(paramsFiltered.row).length > 0) {
				await Store.dispatch('Cliente/setSlacliente', paramsFiltered.row);
				this.slaclienteGridCurrentRow = paramsFiltered.row;
				const { datasource } = OMInstance.getInstanceTekGrid('slaclienteAlertGrid');
				datasource.filter.CDCLIENTE = paramsFiltered.row.CDCLIENTE;
				datasource.filter.CDGRUPFASE = paramsFiltered.row.CDGRUPFASE;
				datasource.filter.CDASSUNTO = paramsFiltered.row.CDASSUNTO;
				datasource.filter.CDCOMPLEXID = paramsFiltered.row.CDCOMPLEXID;
				try {
					await datasource.get();
					this.alertGridDisabled = false;
				} catch (e) {
					OmDialogService.showErrorBackend(e);
				}
			}
		} else {
			this.alertGridDisabled = true;
		}
	}

	/** methods */

	public resetSlaAlertGrid() {
		const alertGridInstance = OMInstance.getInstanceTekGrid('slaclienteAlertGrid');
		if (alertGridInstance.datasource) {
			alertGridInstance.datasource.data = [];
			this.alertGridDisabled = true;
		}
	}

	/** End Section */

	/** *************************************************************************************
	 *************************** Section Add Modal SLA Cliente ******************************
	 ************************************************************************************** */

	/** Modal Methods */

	private addSlaclienteModal!: Modal;

	public async createAddSlaclienteModal() {
		this.addSlaclienteModal = await ModalService.createFromJson('parametrizacaoSlaCliente/modal/addSlacliente', true);
	}

	public destroyAddSlaclienteModal() {
		this.addSlaclienteModal.destroy();
	}

	public openAddSlaclienteModal() {
		this.addSlaclienteModal.show();
	}

	public closeAddSlaclienteModal() {
		this.addSlaclienteModal.hide();
	}

	/** Form|Inputs Methods */

	public async saveAddSlaclienteModal() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteModalForm');
		const { CDCLIENTE } = Store.getters['Cliente/getCliente'];
		const row = { ...form.value, CDCLIENTE };
		if (form.validate()) {
			ScreenService.showLoader();
			try {
				await ApiSlacliente.saveSlacliente(row);
				this.closeAddSlaclienteModal();
				ScreenService.hideLoader();
				OmDialogService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
				await this.reloadSlaclienteGrid();
				this.resetSlaAlertGrid();
			} catch (e) {
				ScreenService.hideLoader();
				OmDialogService.showErrorBackend(e);
			}
		}
	}

	public async cdassuntoOnChangeAddSlaclienteModalForm() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteModalForm');
		const cdassunto = form.getChildInstance<Select>('CDASSUNTO');
		if (this.validationOfValuesAlreadyRegisteredInCdassuntoAndCdgrupfase(form.value)) {
			await cdassunto.setValue('');
			OmDialogService.showError(Messages.ERROR_EQUAL_SLA_PARAMETERS);
		}
	}

	public async cdgrupfaseOnChangeAddSlaclienteModalForm() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteModalForm');
		const idprazosla = form.getChildInstance<Select>('IDPRAZOSLA');
		const cdgrupfase = form.getChildInstance<Select>('CDGRUPFASE');
		if (this.validationOfValuesAlreadyRegisteredInCdassuntoAndCdgrupfase(form.value)) {
			await cdgrupfase.setValue('');
			OmDialogService.showError(Messages.ERROR_EQUAL_SLA_PARAMETERS);
			return;
		}
		if (form.value.CDGRUPFASE === '13') {
			await idprazosla.setValue('1');
			idprazosla.readonly = true;
		} else {
			idprazosla.readonly = false;
		}
	}

	private validationOfValuesAlreadyRegisteredInCdassuntoAndCdgrupfase(formValues: any) {
		const { datasource: datasourceGrid } = OMInstance.getInstanceTekGrid('slaclienteGrid');
		return datasourceGrid.data.find((row) => row.CDASSUNTO === formValues.CDASSUNTO && row.CDGRUPFASE === formValues.CDGRUPFASE);
	}

	public async idprazoslaOnChangeAddSlaclienteModalForm() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteModalForm');
		const qtdiassla = form.getChildInstance<TextInput>('QTDIASSLA');
		if (form.value.IDPRAZOSLA !== '1') {
			qtdiassla.value = '';
			qtdiassla.readonly = true;
		} else {
			qtdiassla.readonly = false;
			qtdiassla.validations.required = {};
		}
	}

	/** *************************************************************************************
	 *************************** Section Grid Sla Cliente Alert ****************************
	 ************************************************************************************** */

	/** Accessors */

	get heightSlaclienteAlertGrid() {
		const aux = ((window.innerHeight * 0.47) - 42) - 178;
		if (aux < 100) {
			return '90px';
		}
		return `${aux}px`;
	}

	set heightSlaclienteAlertGrid(_value: any) {
		// do nothing ();
	}

	public isEditableColStructureTypeId(params: any) {
		if (params.component) {
			const cellAlert = Metadata.getInstance<Select>(`ID_RESP_PROD_ESTRUT_editable_${params.row.ID}`);
			const cellStructure = Metadata.getInstance<Select>(`STRUCTURE_TYPE_ID_editable_${params.row.ID}`);
			if (params.component.value === 'P' || !params.component.value) {
				cellStructure.value = '';
				cellStructure.disabled = true;
			} else if ((cellAlert.value === 'P' || !cellAlert.value) && params.column.name === 'STRUCTURE_TYPE_ID') {
				cellStructure.value = '';
				cellStructure.disabled = true;
			} else {
				cellStructure.disabled = false;
			}
		}
	}

	/** Actions  */

	public async reloadSlaclienteAlertGrid() {
		const { datasource } = OMInstance.getInstanceTekGrid('slaclienteAlertGrid');
		try {
			await datasource.get();
		} catch (e) {
			OmDialogService.showErrorBackend(e);
		}
	}

	public async saveSlaclienteAlertGrid(params: any) {
		params.event.preventDefault();
		try {
			await ApiSlaclienteAlert.updateSlaclienteAlert(params.component.getEditedRows());
			OmDialogService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
			params.component.editing = false;
			params.component.datasource.get();
		} catch (error) {
			OmDialogService.showErrorBackend(error);
		}
	}

	public async deleteSlaclienteAlertGrid(params: any) {
		params.event.preventDefault();
		if (this.slaDefaultClient) {
			const paramsAux = { data: params.component.selectedRows };
			try {
				await ApiSlaclienteAlert.deleteSlaclienteAlert(paramsAux);
				OmDialogService.showSuccess(Messages.SUCCESS_DELETE_DEAFULT);
				await this.reloadSlaclienteAlertGrid();
				params.component.selectedRows = [];
			} catch (e) {
				OmDialogService.showErrorBackend(e);
			}
		} else {
			DialogService.show({
				name: 'error-dialog',
				type: 'error',
				title: 'Erro',
				text: Messages.ERROR_DEFAULT_SLA_ALERT,
			});
		}
	}

	/** End Section */

	/** *************************************************************************************
	 ************************ Section Add Modal SLA Cliente Alert **************************
	 ************************************************************************************** */

	/** Modal Methods */

	private addSlaclienteAlertModal!: Modal;

	public async createAddSlaclienteAlertModal() {
		this.addSlaclienteAlertModal = await ModalService.createFromJson('parametrizacaoSlaCliente/modal/addSlaclienteAlert', true);
	}

	public destroyAddSlaclienteAlertModal() {
		this.addSlaclienteAlertModal.destroy();
	}

	public openAddSlaclienteAlertModal() {
		this.addSlaclienteAlertModal.show();
	}

	public closeAddSlaclienteAlertModal() {
		this.addSlaclienteAlertModal.hide();
	}

	/** Form|Inputs Methods */

	public async idRespProdEstrutChangeAddSlaclienteModalForm() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteAlertModalForm');
		const structureTypeId = form.getChildInstance<Select>('STRUCTURE_TYPE_ID');
		if (form.value.ID_RESP_PROD_ESTRUT !== 'E') {
			structureTypeId.removeValidation('required');
			await structureTypeId.setValue('');
			structureTypeId.disabled = true;
		} else {
			structureTypeId.addValidation('required');
			structureTypeId.disabled = false;
		}
	}

	public async saveAddSlaclienteAlertModal() {
		const form = OMInstance.getInstanceZdForm('addSlaclienteAlertModalForm');
		const { CDCLIENTE } = Store.getters['Cliente/getCliente'];
		const { CDGRUPFASE, CDASSUNTO, CDCOMPLEXID } = Store.getters['Cliente/getSlacliente'];
		const row = { ...form.value, CDCLIENTE, CDGRUPFASE, CDASSUNTO, CDCOMPLEXID };
		if (form.validate()) {
			ScreenService.showLoader();
			try {
				await ApiSlaclienteAlert.saveSlaclienteAlert(row);
				this.closeAddSlaclienteAlertModal();
				ScreenService.hideLoader();
				OmDialogService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
				await this.reloadSlaclienteAlertGrid();
			} catch (e) {
				ScreenService.hideLoader();
				OmDialogService.showErrorBackend(e);
			}
		}
	}
}
